#!/usr/bin/perl
use strict;
use warnings;
use lib do { __FILE__ =~ m|^(.*?)[^/]*$|; "$1"; };

sub rad_profile_parse {

    # Purpose: parse a radial density profile file generated by the
    # program imgstats and determine reasonable values for the inner
    # and outer radii of the capsid. Note that records starting with
    # "#" are skipped and the data lines are always assumed to have
    # the following format, where the additional fields may be subject
    # to change
    #
    # r  n  avg  std  [additional fields]
    #
    # r    - radius of annulus (pixels)
    # n    - number of voxels in radial shell
    # avg  - average voxel value
    # std  - standard deviation of voxel values
    #
    # Input parameters:
    #   $infile  - file containing radial density profile
    #
    # Return values:
    #   $rad_in  - inner capsid radius
    #   $rad_out - outer capsid radius

    my ($infile) = @_;

    my $date = localtime();

    my @avg = ();
    my @avg3pt = ();
    my @std = ();
    my @ravgstd = ();
    my @ravg = ();
    my @nstd = ();
    my ($ravgstd_maxval, $ravgstd_maxloc, $count) = (0, 0, 0);
    my ($nstd_maxval, $nstd_maxloc) = (0, 0);
    my ($ravg_maxval, $std_maxref) = (0, 0);
    my ($rad_in, $rad_out);
    my ($rad_in_msk, $rad_out_msk) = (1, 1);

    # Loop through records and find maximum value of r*avg*std and sqrt(n)*std

    open(INSTAT, $infile);
    while(<INSTAT>) {
	if (/\s*#/) {
	    next;
	}

	s/^\s+//;
	my (undef, $nvxl, $avg, $std) = split(/\s+/);
	unless(defined $std) {
	    next;
	}
	
	my $ravgstd = $count*$avg*$std;
	my $ravg = $count*$avg;
	my $nstd = sqrt($nvxl)*$std;
	
	push(@avg, $avg);
	push(@std, $std);
	push(@ravgstd, $ravgstd);
	push(@ravg, $ravg);
	push(@nstd, $nstd);

	if($ravgstd > $ravgstd_maxval) {
	    $ravgstd_maxval = $ravgstd;
	    $ravgstd_maxloc = $count;
	}
	if($ravg > $ravg_maxval) {
	    $ravg_maxval = $ravg;
	    $std_maxref= $std;
	}

	$count++;
    }
    
    # Get moving 3-point average of avg annulus value
    for (my $i=2; $i<$count-1; $i++) {
	$avg3pt[$i] = ($avg[$i-1] + $avg[$i] + $avg[$i+1])/3.0;
    }

    # Estimating outer capsid radius: Method 1
    # Starting from exterior of particle, look for first point
    # where avg*std exceeds 1/10th maximum value.

    $rad_out = $count-1;
    for (my $i=$count-1; $i>$ravgstd_maxloc; $i--) {
	if ($ravgstd[$i] > $ravgstd_maxval*0.1) {
	    $rad_out = $i;
	    last;
	}
    }
    # RSS kludge - scale rad_out by 5%
    $rad_out *= 1.05;
    if ($rad_out > $count-1) {$rad_out = $count-1;}

    # Estimating outer capsid radius for masking
    # Starting from radius of max r*avg*std, look for first point
    # where std drops below 1/4th of value at max r*avg.

    $rad_out_msk = $ravgstd_maxloc;
    for (my $i=$ravgstd_maxloc; $i<$count; $i++) {
	if ($std[$i] < $std_maxref*0.25) {
	    $rad_out_msk = $i;
	    last;
	}
    }
    if ($rad_out_msk < $rad_out) { $rad_out_msk = $rad_out;} 

    my $rad_in_1 = 1;
    my $rad_in_2 = 1;

    # First estimate of inner capsid radius: Start from maxloc and
    # search inward until average drops to avg[maxloc]/2

    for (my $i=$ravgstd_maxloc; $i>0; $i--) {
	if ($avg[$i] < $avg[$ravgstd_maxloc]/2) {
	    $rad_in_1 = $i;
	    last;
	}
    }

    # Second estimate of inner capsid radius: Start from maxloc and
    # search inward until dip found in 3-point average of avg annulus
    # pixel value

    my $reached_peak = 0;
    for (my $i=$ravgstd_maxloc; $i>2; $i--) {
	if (!$reached_peak && $avg3pt[$i-1] < $avg3pt[$i]) {
	    $reached_peak = 1;
	}
	if($reached_peak && $avg3pt[$i-1] > $avg3pt[$i]) {
	    $rad_in_2 = $i;
	    last;
	}
    }

    if($rad_in_1 > $rad_in_2) {
	$rad_in = $rad_in_1;
    } else {
	$rad_in = $rad_in_2;
    }
	
    # Estimate of inner capsid radius for masking: Start from maxloc and
    # search inward until std drops below half the value at max r*avg
    for (my $i=$ravgstd_maxloc; $i>1; $i--) {
	if ($std[$i] < $std_maxref * 0.5) {
	    $rad_in_msk = $i;
	    last;
	}
    }
    if ($rad_in_msk > $rad_in) { $rad_in_msk = $rad_in;} 
    # Find the maximum sqrt(n)*stad_dev, starting from rad_in
    # This is the new capsid score
    for (my $i=$rad_in; $i<$count-1; $i++) {
	   if ($nstd[$i] > $nstd_maxval) {
	     $nstd_maxval = $nstd[$i];
	     $nstd_maxloc = $i;
	   }
    }

    if (fileno(OUT)) {
        print OUT "Parsing imgstats radial density file\n";
        print OUT "   reached_peak = $reached_peak\n";
        print OUT "   rad_in_1    = $rad_in_1\n";
        print OUT "   rad_in_2    = $rad_in_2\n";
        print OUT "   rad_in      = $rad_in\n";
        print OUT "   rad_in_msk  = $rad_in_msk\n";
        print OUT "   rad_out     = $rad_out\n";
        print OUT "   rad_out_msk = $rad_out_msk\n";
        print OUT "\n";
    }
    close(INSTAT);

# previous metric
#    return($rad_in, $rad_out, $ravgstd_maxval);
    return($rad_in, $rad_out, $rad_in_msk, $rad_out_msk, $ravgstd_maxloc, $nstd_maxval);
}
1;
