use strict;
use warnings;
use lib do { __FILE__ =~ m|^(.*?)[^/]*$|; "$1"; };

use get_info;

# need to pass array and index to last line read
sub stack_get_next {

    # Purpose: scan through the content of a loaded parameter file
    #          and extract all the lines related to a single stack
    #
    # Input arguments:
    #   $istart     - starting index for scanning parameter lines
    #   @array      - array of lines loaded from a parameter file
    #
    # Return values:
    #   $idx         - index of first line after stack
    #   $stack_name  - string line containing stack file name
    #   $stack_pars  - string line with stack parameters
    #   @stack_parts - array of particle parameter lines

    my ($istart,@array) = @_;
   
    my $item;
    my @parts = ();
    for my $i ($istart .. $#array) {
      $item = $array[$i];
      if($item =~ /\S/){
        my @p = split ' ', $item;
        if(scalar(@p) < 5) {
          my $stack_name = $p[0];
          my $stack_pars = $array[$i+1];
          for my $ii ($i+2 .. $#array) {
            $item = $array[$ii];
            if($item =~ /\S/){
              my @p = split ',', $item;
              if(scalar(@p) > 8) {
                push(@parts,$item);
              }
              else {
                return ( $ii, $stack_name."\n", $stack_pars, @parts );
              }
            }
          }
          return ( $#array, $stack_name."\n", $stack_pars, @parts );
        }
      }
    }
    return;
}

sub stack_make_parts_params {

    # Purpose: scan through the content of a loaded parameter file
    #          and extract all the lines related to a single stack
    #
    # Input arguments:
    #   $stack_file  - name of particles stack file
    #
    # Return values:
    #   @stack_parts - array of particle parameter lines

    my ($stack_file) = @_;

    chomp($stack_file);

    my $numimg = get_numimages($stack_file);
	my ($nx, $ny, $nz) = get_dimensions($stack_file);
	my $ox = int($nx/2)+1;
	my $oy = int($ny/2) +1;
    my @parts = ();
    my $item;
    
    foreach my $i (1..$numimg) {
      $item = sprintf("%7d,   0.000,   0.000,   0.000,%8.2f,%8.2f, 1.000, 0.000, 0.000, 0.000\n",$i,$ox,$oy);
      push(@parts,$item);
    }

    return (@parts);
}

sub stack_files_join {

    # Purpose: join content of a list of files into a single one
    #
    # Input arguments:
    #   $outfile  - name of output particles stack file
    #   @files    - array of input files
    #
    # Return values:
    #   none

    my ($outfile,@files) = @_;
     
    # Write to a temporary file because the output file
    #  could be one of the input files too
    my $tmpoutfile = '_autopp_tmp.txt';
    open(OUT, ">$tmpoutfile");

    foreach my $datfile (@files) {
      open(IN, $datfile);
      print "Processing file $datfile\n";

      while(<IN>) {
        my $line = $_;
        print OUT "$line";
      }
      close(IN);
    }
    close(OUT);
    move $tmpoutfile, $outfile;
}

sub stack_files_eosplit {
     
    # Purpose: split content of a each from a list of files
    #          into two files, separating even- and odd-
    #          numbered particels
    #          It is assumed that each files either ends with 
    #          'dat' or 'dat_XXXX'. If not, the result is
    #          undefined
    #
    # Input arguments:
    #   @files    - array of input files
    #
    # Return values:
    #   @eofiles  - array of generated files
    
    my (@files) = @_;
    
    my @eofiles = ();
    my ($stack_name, $stack_pars, @stack_parts);
    
    # split each parameter file
    foreach my $datfile (@files) {

      open my $infile, '<', $datfile or die "Can't open $datfile!";
      my @infile = <$infile>;  #each line of the file into an array
      close $infile or die "Can't close $infile!";

      my ($prefix, $suffix) = ($datfile =~ /(.*)_(.*)/);
      unless (defined $prefix) {
       $prefix = $datfile;
      }
      # special case: file ends with 'dat'
      my $last3chars = substr($datfile,-3);
      if ($last3chars eq 'dat') {
        $prefix = $datfile;
        $suffix = '';
      } 

      my $ofile = "${prefix}o_";
      my $efile = "${prefix}e_";
      if (defined $suffix) {
        $ofile = "${prefix}o_${suffix}";
        $efile = "${prefix}e_${suffix}";
      }
      open my $outfile_odd, '>', "$ofile" or die "Can't open $ofile!";
      open my $outfile_even, '>', "$efile" or die "Can't open $efile!";

      my $is = 0;
      my $nestacks = 0;
      my $nostacks = 0;

      while (($is, $stack_name, $stack_pars, @stack_parts) = stack_get_next($is,@infile)) {
        
        my @olist = ();
        my @elist = ();
        push(@olist,$stack_name);
        push(@olist,$stack_pars);
        push(@elist,$stack_name);
        push(@elist,$stack_pars);

        if (!@stack_parts) {
          print "WARNING: no particles found for $stack_name";
          print "         generating them from the stack\n";
          @stack_parts = stack_make_parts_params($stack_name);
        }
        my $nep = 0;
        my $nop = 0;
        foreach (@stack_parts) {
           my $pid = substr((split(' ',$_))[0],0,-1);
           if ($pid % 2) {
            push(@olist,$_);
            $nop = $nop + 1;
           }
           else {
            push(@elist,$_);
            $nep = $nep + 1;
           }
        }

        if ($nop>0) {
          foreach (@olist) {
            print $outfile_odd $_;
          }
          $nostacks = $nostacks + 1;        
        }
        if ($nep>0) {
          foreach (@elist) {
            print $outfile_even $_;
          }
          $nestacks = $nestacks + 1;        
        }
      }
      close $outfile_odd, $outfile_even;
      if ($nostacks>0) {
        push(@eofiles,$ofile);
      }
      else {
        unlink $ofile;
      }
      if ($nestacks>0) {
        push(@eofiles,$efile);
      }
      else {
        unlink $efile;
      }
    }
     
    return (@eofiles);
}
1;

	




